/*
This code shows Data Output, Raw Data Output, Minimum Calibration and Maximum Calibration value of RKI-4421 soil moisture Sensor on Serial LCD Display.

Connections of Sensor, Arduino and MAX485 TTL to RS485 Converter
Serial Port 0 is not used to connect to drive because its connected to USB-Serial and used to show information on console.

For Arduino Uno Software serial needs to be used as there is only one hardware serial port and its connected to USB-Serial. 
   Sensor       MAX485 TTL to RS485 Converter    Arduino (Software Serial)
     A                 A                         -
     B                 B                         -
    GND               GND                        GND
    5V                 -                         5V
    -                  R0                        D2
    -                  DE & RE                   D8
    -                  DI                        D3

Connections of Serial LCD Display and Arduino https://robokits.co.in/displays/serial-16x2-character-lcd 
    Serial LCD Display                          Arduino
           VCC                                    5V
           GND                                    GND
           RXD                                    TX
   
For more information of soil sensor see :https://robokits.co.in/sensors/water-moisture/soil-moisture-sensor-with-rtu-modbus-communication
*/

#include <ModbusMaster.h>   
#include <SoftwareSerial.h> 
SoftwareSerial mySerial(2,3);    //Software Serial port For Arduino Uno
#define MODBUS_DATA_TRANSACTION_PIN 8   //DE & RE Pin of MAX485
ModbusMaster node;                      //object for ModbusMaster

short a1,a2,a3,a4;
uint8_t Slave_ID = 1;   //Slave ID of the Sensor

void setup() 
{
    pinMode(MODBUS_DATA_TRANSACTION_PIN, OUTPUT);
    digitalWrite(MODBUS_DATA_TRANSACTION_PIN, 0);
    Serial.begin(9600);     //Baud rate is 9600 8N1
    mySerial.begin(9600);   //Modbus Baud rate is 9600 8N1
    node.begin(mySerial);
    node.preTransmission(preTransmission);
    node.postTransmission(postTransmission);
}

void loop() 
{
   a1 = node.Data_Output(Slave_ID);    
   a2 = node.Raw_Data_Output(Slave_ID);  
   a3 = node.Minimum_Calibration_Value(Slave_ID);
   a4 = node.Maximum_Calibration_Value(Slave_ID);

   clearLCD();
   Serial.print("Data:");
   Serial.print(a1);  
   Serial.print(" Raw:");
   Serial.print(a2); 
   newLine();  
   Serial.print("Min:");
   Serial.print(a3);  
   Serial.print(" Max:");
   Serial.print(a4);
   delay(20);
}

void preTransmission() 
{
  digitalWrite(MODBUS_DATA_TRANSACTION_PIN, 1);
}

void postTransmission()
{
  digitalWrite(MODBUS_DATA_TRANSACTION_PIN, 0);
}

void clearLCD()
{
  Serial.write(12);
}
void newLine() 
{ 
  Serial.write(13); 
}
